#!/usr/bin/env python
#
# a python equivalent to the perl CertLib.pm
#
#package CertLib;
#use strict;
#use LWP::UserAgent;

import urllib
from M2Crypto import RSA, X509, EVP, m2, Rand, Err
#from M2Crypto import RSA, EVP, m2, Rand, Err
#import X509

def getversion():
    return pkgversion

def generateRSAKey():
    return RSA.gen_key(2048, m2.RSA_F4)

def makePKey(key):
    pkey = EVP.PKey()
    pkey.assign_rsa(key)
    return pkey
    
def makeRequest(pkey,host,service=''):
    req = X509.Request()
    # Seems to default to 0, but we can now set it as well, so just API test
    req.set_version(req.get_version())
    req.set_pubkey(pkey)
    # req.set_pubkey(EVP.PKey(req.get_pubkey()))# Just a test of the API
    name = X509.X509_Name()
    name.DC = 'org'
    name.DC = 'doegrids'
    name.OU = 'Services'
    if service :
        service = service + '/'
    name.CN = service + host
    req.set_subject_name(name)
    # ext1 = X509.new_extension('subjectAltName', 'DNS:foobar.example.com')
    # ext2 = X509.new_extension('nsComment', 'Hello there')
    # extstack = X509.X509_Extension_Stack()
    # extstack.push(ext1)
    # extstack.push(ext2)

    # assert(extstack[1].get_name() == 'nsComment')
    
    # req.add_extensions(extstack)
    req.sign(pkey, 'sha1')
    return req

def genServiceCSR(reqfile,keyfile,host,service='',encrypt=''):
    """
    generate a PKCS10 certificate signing request

    @type reqfile: string
    @param reqfile: name of file to save CSR in PEM format

    @type keyfile: string
    @param keyfile: name of file to save private key in PEM format

    @type host: string
    @param host: FQDN of host in the subject of the certificate

    @type service: string
    @param service: optional name of service on the specified host

    @type encrypt: string
    @param encrypt: any value will cause private key to be encrypted.
    There is an interactive prompt for the passphrase for encryption.
    If you omit the argument or pass a null string the private key is
    stored in the clear.

    Subject names are of the form
    CN=[<service>/]host,OU=Services,DC=doegrids,DC=org
    """
    rsa = generateRSAKey()
    pkey = makePKey(rsa)
    req = makeRequest(pkey,host,service)
    req.save_pem(reqfile)
    if encrypt :
        rsa.save_key(keyfile,'aes_256_cbc')
    else :
        rsa.save_key(keyfile,None)
    return req

def cncheck(icn):
    """
    checks a string if it seems valid for a CN and returns
    CN, host, service

    @type icn: string
    @param icn: possible CN string
    """

    #strip any leading '*='
    ieq = icn.find("=")
    if ieq > -1:
        icn = icn[ieq+1:]
    islash = icn.find("/")
    if islash > -1:
        service = icn[0:islash]
        fqdn = icn[islash+1:]
    else:
        service = ""
        fqdn = icn
    dots = fqdn.count(".")
    if dots == 0:
        print "W no . in " + fqdn
    return icn, fqdn, service

def getFQDNfromCert(certfile):
    """
    get FQDN and service elements of a service certificate CN

    @type certfile: string
    @param certfile: path name to certificate file
    """
    cert = X509.load_cert(certfile)
    subject = cert.get_subject()
    ans = subject.as_text()
    cn = ans[ans.find("CN=")+3:]
    # print ans + " CN " + cn
    ocn, fqdn, service = cncheck(cn)
    return fqdn, service
    
def genCSRfromCert(reqfile,keyfile,certfile,encrypt=''):
    """
    generate a CSR from a certificate file

    @type reqfile: string
    @param reqfile: path name to CSR file in PEM format

    @type keyfile: string
    @param keyfile: path name to private key file
    
    @type certfile: string
    @param certfile: path name to public certificate in PEM format

    @type encrypt: string
    @param encrypt: any value will cause private key to be encrypted.
    There is an interactive prompt for the passphrase for encryption.
    If you omit the argument or pass a null string the private key is
    stored in the clear.
    """
    cert = X509.load_cert(certfile)
    subject = cert.get_subject()
    # print subject.as_text()
    rsa = generateRSAKey()
    pkey = makePKey(rsa)
    req = X509.Request()
    req.set_version(req.get_version())
    req.set_pubkey(pkey)
    req.set_subject_name(subject)
    req.sign(pkey, 'sha1')
    req.save_pem(reqfile)
    if encrypt :
        rsa.save_key(keyfile,'aes_256_cbc')
    else :
        rsa.save_key(keyfile,None)
    return req    
    

def get_osgvos():
    return OSGVOS

def get_ras():
    return RAS

def get_cas():
    return CA

def load_ras(rasurl):
    defras = {
        'anl' : 'ANL: Argonne National Lab',
        'esg' : 'ESG: Earth System Grid',
        'esnet' : 'ESnet: DOE Science network',
        'fnal' : 'FNAL: Fermilab certificates',
        'fusiongrid' : 'FusionGRID: National Fusion Grid Project',
        'lbnl' : 'LBNL: Berkeley Lab',
        'lcg' : 'LCG: LHC Computing Grid',
        'nersc' : 'NERSC: computer center, see www.nersc.gov',
        'ornl' : 'ORNL: Oak Ridge National Lab',
        'osg' : 'OSG: Open Science Grid (choose this if nothing else applies)',
        'pnnl' : 'PNNL: Pacific Northwest National Lab'
	}
    ras = {}
    try:
        f = urllib.urlopen(rasurl)
        page = f.read(10000)
        lines = page.split("\n")
        for aline in lines:
            words = aline.split(",")
            if len(words) > 1:
                label = words[0]
                descr = words[1]
                ras[label] = descr
    except:
        print "CertLib.py - could not read current RA list, will use default"
        ras = defras
    return ras

#################### get_osgvolist #####################
def load_vos(url):
    vos = {}
    defvos = {
        'cdf' : 'CDF: Collider Detector at Fermilab',
        'cms' : 'CMS: Compact Muon Solenoid',
        'compbiogrid' : 'CompBioGrid: Computational Cell Biology',
        'des' : 'DES: Dark Energy Survey',
        'dosar' : 'DOSAR: Distributed Organization for Scientific and Academic Research',
        'dzero' : 'DZero: D0 Experiment',
        'engage' : 'Engage: OSG Engagement Activity',
        'fermilab' : 'Fermilab: The Fermilab VO',
        'fmri' : 'fMRI: Functional Neuroimaging Community, Dartmouth Brain Imaging Center, and Dartmouth Community',
        'gadu' : 'GADU: Genome Analysis and Database Update',
        'geant4' : 'geant4: Simulation toolkit for high energy physics experiments and biomedical applications',
        'glow' : 'GLOW: Grid Laboratory of Wisconsin',
        'gpn' : 'GPN: Great Plains Network',
        'grase' : 'GRASE: Grid Resources for Advanced Science and Engineering',
        'gridex' : 'GridEx: Grid Exerciser',
        'grow' : 'GROW: Grid Research and educatiOn group @ IoWa',
        'gugrid' : 'GUGrid: Georgetown University Grid',
        'i2u2' : 'i2u2: Interactions in Understanding The Universe',
        'ivdgl' : 'iVDGL: International Virtual Data Grid Laboratory',
        'ligo' : 'LIGO: Laser Interferometer Gravitational Wave Observatory',
        'mariachi' : 'MARIACHI: Mixed Apparatus for Radar Investigation of Cosmic-rays of High Ionization',
        'mis' : 'MIS: Monitoring Information System',
        'nanohub' : 'nanoHUB: Online simulations and more',
        'nwicg' : 'NWCIG: Northwest Indiana Computational Grid',
        'osg' : 'OSG: Open Science Grid VO for small groups and individuals',
        'osgedu' : 'OSGEDU: OSG VO for educational grid workshops',
        'sdss' : 'SDSS: Sloan Digital Sky Survey',
        'star' : 'STAR: The STAR Experiment',
        'usatlas' : 'USATLAS: ATLAS in US (A Toroidal Lhc ApparatuS)'
	}
    inst_vos = {
        'bnl' : 'BNL: Brookhaven lab researchers not in an OSG registered VO',
        'jlab' : 'JLab: Jefferson Lab researchers',
        'slac' : 'SLAC: Stanford Linear Accelerator Center researchers',
	}
    vos = inst_vos
    try:
        f = urllib.urlopen(url)
        page = f.read(10000)
        osgvos = page.split("<br>")
        for avo in osgvos:
            vd = avo.split("||")
            if len(vd) > 1:
                vo = vd[0]
                descr = vd[1]
                vo = vo.strip()
                descr = descr.strip()
                vos[vo] = descr
    except:
        print "CertLib.py - could not read list of VOs, will use default"
        vos = defvos
        vos.update(inst_vos)
        
    if not vos['osg']:
	vos = defvos
        vos.update(inst_vos)

    return vos

# initialize

pkgversion = "2-3"


################################################################################
# stuff for the OSG cert-scripts package in VDT
################################################################################

pkgversion = "V2-3"

RAS = load_ras("http://www.ppdg.net/RA/communities/doegrids-ras.txt")

OSGVOS = load_vos("http://www.grid.iu.edu/osg-includes/vos.php")
    

doegridsca = {
    'addr' : 'pki1.doegrids.org',
    'port' : 443,
    'path' : '/enrollment',
    'cpcps' : 'http://www.doegrids.org/Docs/CP-CPS.pdf',
    'retrieve' : 'https://pki1.doegrids.org/displayBySerial?op=displayBySerial&serialNumber='
    }
testca = {
    'addr' : 'amber.es.net',
    'port' : 443,
    'path' : '/enrollment',
    'cpcps' : 'none',
    'retrieve' : 'https://amber.es.net/displayBySerial?op=displayBySerial&serialNumber='
    }
testca2 = {
    'addr' : 'copper.es.net',
    'port' : 1082,
    'path' : '/enrollment',
    'cpcps' : 'none',
    'retrieve' : 'http://copper.es.net:1082/displayBySerial?op=displayBySerial&serialNumber='
    }
ppdgca = {
    'addr' : 'www.ppdg.net',
    'port' : 443,
    'path' : '/cgi-bin/ssl/test4567.cgi',
    'cpcps' : 'none',
    'retrieve' : 'https://www.ppdg.net/cgi-bin/ssl/outofluck'
    }
CA = { 'doegrids' : doegridsca,
        'testca' : testca,
        'test2' : testca2,
        'ppdg' : ppdgca }
