#!/usr/bin/env python
#

#import csrUtils
import StringIO
import urllib
from socket import getfqdn, gethostbyname
from M2Crypto import SSL, httpslib
import M2Crypto
import getopt, sys, os
import fileinput
import CertLib

def reqPrep(email,affiliation,vo,req='',reqfile=''):
    """
    prepare the POST contents for a PKCS10 CSR

    @type req: X509 Request
    @param req: PKCS10 CSR

    @type reqfile: string
    @param reqfile: path to CSR in PEM format

    @type email: string
    @param email: email address to submit with CSR

    @type affiliation: string
    @param affiliation: registration authority

    @type vo: string
    @param vo: virtual organization

    returns the URL encoded contents for the HTTP POST to the CA
    """
    
    ###########################################################
    # Hidden form fields - These are all defaults
    # Standard Parameters required by the Service 
    #  ********** DO NOT CHANGE THESE VALUES ******************
    ###########################################################
    exechost = getfqdn()
    execaddr = gethostbyname(exechost)
    HIDDEN  = "requestFormat=clientAuth&"
    HIDDEN += "doSslAuth=on&certauthEnroll=true&"
    HIDDEN += "certauthEnrollType=single&certType=client&"
    HIDDEN += "ssl_client=true&ssl_server=true&digital_signature=true&"
    HIDDEN += "AuthServerRequest=true&non_repudiation=true&"
    HIDDEN += "key_encipherment=true&uid=doegridsuser&pwd=doegridsuser&"
    HIDDEN += "RemoteAddr=" + execaddr + "&"
    HIDDEN += "RemoteHost=" + exechost + "&CHECKFLAG=&CRMFRequest=&"
    HIDDEN += "cmmfResponse=true&authenticator=UidPwdDirAuth&"
    HIDDEN += "email=true"
    HIDDEN = urllib.quote_plus(HIDDEN,'=&')

    if vo :
        faffil = affiliation + ":" + vo
    else :
        faffil = affiliation
    FORMFIELDS = "csrRequestorEmail=" + email + "&"
    FORMFIELDS += "csrRequestorAffiliation=" + faffil + "&"
    FORMFIELDS += "csrRequestAffiliation=" + affiliation + "&"
    FORMFIELDS += "csrVirOrgName=" + vo + "&"
    FORMFIELDS += "submit=Submit"
    FORMFIELDS = urllib.quote_plus(FORMFIELDS,'=&')

    if req:
        CR = "requestFormat=pkcs10&pkcs10Request=" + req.as_pem()
    elif reqfile:
        f = open(reqfile,'r')
        CR = "requestFormat=pkcs10&pkcs10Request=" + f.read()
    else:
        print "reqPrep failed to get request"
        
    CR = urllib.quote_plus(CR,'=&')

    content = HIDDEN + "&" + CR + "&" + FORMFIELDS

    if not quiet:
        print content
    return content


def read_csv(infile):
    lines = []
    lineno = 0
    label = 'none'
    for aline in fileinput.input(infile):
        aline = aline.rstrip("\n\r")
        lineno = lineno + 1
        words = aline.split(",")
        if not quiet:
            for w in words:
                print '%4d  %s' % (lineno, w)
        if aline[0:0] == '#':
            continue
        if lineno == 1:
            # check if first line is header by looking for keywords
            lline =aline.lower()
            lwords = lline.split(",")
            iscn = 0
            iscsr = 0
            iscert = 0
            col = 0
            iw = 0
            for lw in lwords:
                iw = iw + 1
                if lw == "cn":
                    iscn = 1
                    col = iw
                if lw == "csr":
                    iscsr = 1
                    col = iw
                if lw == "certfile":
                    iscert = 1
                    col = iw
            ok = iscn + iscsr + iscert
            # if one keyword found then use it as header, otherwise
            # use it as normal line
            if ok == 1:
                col = col - 1
                label = words[col]
                continue
        if len(words) > col:
            lines.append(words[col])
    return label, lines
        
                

def usage():
    print """
    cert-gridadmin.py

    is used to generate CSRs (certificate signing requests)
    and to submit the CSRs to the CA for generating certificates.
    The submission is authenticated with the client certificate which
    has been pre-authorized to submit via the GridAdmin interface.
    It functions in four basic modes called csr-from-list, csr-from-cert,
    csr-submit and cn-submit, as described below, in addition to
    help and information.  Options and arguments are not order dependent
    except that the mode label listeed above should be the first non-option
    argument.

    cert-gridadmin.py [-h | -l | -n] [anything]

    cert-gridadmin.py csr-from-list [-n] [-d wdir] [-i file-with-list]
                      [CN1 [CN2 [CN3 ...]]]

    cert-gridadmin.py csr-from-cert [-d wdir] [-i file-with-list] [cert1
                     [cert2 [cert3 ...]]]

    cert-gridadmin.py csr-submit -e email -a RA -v VO [-t] [-c creddir] [-s CA]
                 [-d wdir] [-i file-with-list] [CSR1 [CSR2 [CSR3 ...]]]

    cert-gridadmin.py cn-submit -e email -a RA -v VO [-t] [-c creddir] [-s CA]
                 [-d wdir] [CN1 [CN2 [CN3 ...]]]

      Options:
         -a RA is the RA affiliaiton
         -c creddir is the directory containing the users credentials,
                    usercert.pem and userkey.pem.  The default value
                    is ~/.globus. See also -p, -k.
         -d wdir is the working directory used for the request, key and
                 certificate files. It does not apply to -i.
         -e email is the email address submitted with the CSRs to be included
                  in the issued certificates
         -h print out usage (and exit)
         -i file-with-list is a path to the file (CSV format) containing
                           the list of items. See descriptions of modes
                           below for the content of this file.
         -k privkey-file is the name of the private key file of the users
                         credentials. The full path is creddir/privkey-file.
                         See also -c, -p.
         -l list valid choices for (-a RA), (-s CA), (-v VO), and exit
         -n more verbose output (Not quiet).
         -p public-cert-file is the filename of the users public certificate.
                             The full path is creddir/public-cert-file.
                             See also -c, -k.
         -s CA is the name of the Certification Authority to post the CSR to.
               Only doegrids is a production CA. The others are testing CAs
               or debugging web servers.
               See also -l.
         -t test only mode for cn-submit, csr-submit. Script is executed
            without posting to the CA.
         -v VO is the Virtual Organization name associated with the OSG RA.
               See also -a, -l.
         -x ssl-debug-level sets the level of debug messages for the SSL
                            connection.

    Information only mode:
        See -h, -l.

    csr-from-list mode:
        generates a set of PKCS10 format certificate signing requests
        for the list of CNs supplied, in the namespace
        OU=Services,DC=doegrids,DC-org.  The set of CSRs and private keys
        are located in wdir.  A CN has the form service/fqdn, and the
        corresponding CSRs and private key files are named as
        fdqn-service-req.pem and fqdn-service-key.pem, respectively.
        The contents of the (-i) file-with-list optional file is a
        list of CNs in CSV format. If the optional header line is used
        it should label the appropriate column with 'CN' (not case dependent).
        If not header line is used the values in the first column are used.
        Comment lines have a '#' as the first character.
        CN component names can also be provided as commandline arguments
        as indicated above.
                        
    csr-from-cert mode:
         generates a set of PKCS10 format certificate signing requests
         matching the list of certificates supplied, in the namespace
         OU=Services,DC=doegrids,DC-org.  The set of generated CSRs and private
         keys are located in wdir.  A CN has the form service/fqdn,
         and the corresponding CSRs and private key files are named as
         fdqn-service-req.pem and fqdn-service-key.pem, respectively.
         The optional (-i) file-with-list contains a list of path names
         to certificate files to be used, in CSV format.  If the optional
         header line is used the column should be labelled with 'CERTFILE'
         (not case dependent). If no header line is used the values in column
         one are used.  Comment lines have a '#' as the first character.
         Paths to certificate files can also be provided as command line
         arguments as shown above.

    csr-submit mode:
         submits a set of CSRs to the GridAdmin interface of the
         CA and downloads the corresponding signed certificates.
         It is assumed that CSR files are named as fqdn-service-req.pem
         and the corresponding private key files are named
         fqdn-service-key.pem.  The downloaded certificates are stored
         in files named fqdn-service-cert.pem.
         Possibly, if no specific CSRs are given, it will submit all
         the fqdn-service-req.pem files it can find in wdir.
         The optional (-i) file-with-list contains, in CSV format, a list
         of path names to the CSR files to submit.  If the optional header
         line is used it should label the appropriate column with 'CSR'
         (case independent). If a header line is not used the first column
         is used.  Comment lines have a '#' as the first character.
         Path names to CSR files can also be given on the command line
         as shown above.

    cn-submit mode:
         generates CSRs based on the CNs listed and submits these CSRs
         to the GridAdmin interface of the DOEGrids CA and downloads
         the corresponding signed certificates.
         The CSR files are named as fqdn-service-req.pem
         and the corresponding private key files are named
         fqdn-service-key.pem.  The downloaded certificates are stored
         in files named fqdn-service-cert.pem.
         The optional (-i) file-with-list contains, in CSV format, a list
         of CNs to submit.  If the optional header line is used it should
         label the appropriate column with 'CN' (case independent).
         If a header line is not given the value in the first column is used.
         Comment lines have a '#' as the first character.
         CNs can also be given on the command line as shown above.
         In this mode the private key generation and submission
         to the CA happens within the same command, which can be
         time consuming.

    Note that file-with-list should be in CSV format (RFC4180), although
    this program does not contain a complete implementation of the RFC.
    A # in column one indicates a comment line, in variance with the RFC.
    If a header line is used, the labels CN, CSR and CertFile correspond
    to the values used for csr-from-list, csr-submit and csr-from-cert
    respectively.  Otherwise the interesting value is assumed to
    be in the left most column.

    """
    return

def list_caravos():
    print "\nValid CAs: "
    ck = CAS.keys()
    for ca in sorted(set(ck)):
        print '%12s, CP/CPS=%s' % (ca, CAS[ca]['cpcps'])
    print "\nValid RAs  (affiliation)"
    rk = RAS.keys()
    for ra in sorted(set(rk)):
        print '%12s %s' % (ra, RAS[ra])
    print "\nValid VOs for the OSG RA"
    vk = VOS.keys()
    for vo in sorted(set(vk)):
        print '%12s %s' % (vo, VOS[vo])
    return

def get_cert(data):
    """
    pull the PEM certificate out of the CA html response

    @type data: string
    @param data: the html response from the CA
    """
    begstr = "-----BEGIN CERTIFICATE-----"
    endstr = "-----END CERTIFICATE-----"
    try:
        cbegin = data.index(begstr)
        cend = data.index(endstr,cbegin)
        cend = cend + len(endstr)
        cert = data[cbegin:cend]
    except:
        print "no certificate found"
        print data
    return cert

if __name__ == "__main__":

    RA = ''
    cdir = os.environ['HOME']
    wdir = "."
    CA = "doegrids"
    post = True
    quiet = True
    cnlist = []
    infile = ''
    KEYFILE = 'userkey.pem'
    CERTFILE = 'usercert.pem'
    headers = {"Content-type": "application/x-www-form-urlencoded",
               "user-agent": "CSR-Submit-Script/V0.1"}
    VOS = CertLib.get_osgvos()
    RAS = CertLib.get_ras()
    CAS = CertLib.get_cas()
    ssldebug = 0

    try:
        opts, args = getopt.gnu_getopt(sys.argv[1:],"a:c:d:e:hi:k:lnp:s:tv:x:")
    except getopt.GetoptError:
        usage()
        sys.exit(2)
        
    for o, a in opts:
        if o == "-a":
            RA = a
            if RA not in RAS:
                print "invalid RA " + RA
                list_caravos()
                sys.exit(2)
        if o == "-c":
            cdir = a
        if o == "-d":
            wdir = a
        if o == "-e":
            email = a
        if o == "-h":
            usage()
            sys.exit(1)
        if o == "-i":
            infile = a
        if o == "-k":
            KEYFILE = a
        if o == "-l":
            list_caravos()
            sys.exit(1)
        if o == "-n":
            quiet = False
        if o == "-p":
            CERTFILE = a
        if o == "-s":
            CA = a
            if CA not in CAS:
                print "invalid CA " + CA
                list_caravos()
                sys.exit(2)
        if o == "-t":
            post = False
        if o == "-v":
            VO = a
            if VO not in VOS:
                print "invalid VO " + VO
                list_caravos()
                sys.exit(2)
        if o == "-x":
            ssldebug = a

    if args[0] == "csr-from-list":
        lines = []
        if not os.path.exists(wdir):
            print "directory " + wdir + " does not exist"
            usage()
            sys.exit(2)
        if os.path.exists(infile):
            label, lines = read_csv(infile)
        else:
            print infile + " does not exist, will try command line CNs"
        for cn in args[1:]:
            lines.append(cn)
        for mite in lines:
            miteb = CertLib.cncheck(mite)
            if len(miteb[1]) > 0:
                cnlist.append(miteb)
        for cn in cnlist:
            if not quiet:
                print cn 
            if cn[2]:
                ss = "-" + cn[2]
                sn = cn[2] + "/"
            else:
                ss = ""
                sn = ""
            reqf = wdir + "/" + cn[1] + ss + "-req.pem"
            keyf = wdir + "/" + cn[1] + ss + "-key.pem"
            cns = sn + cn[1]
            print "generating CSR for " + cns
            req = CertLib.genServiceCSR(reqf,keyf,cn[1],cn[2])
            print "wrote CSR=" + reqf + " key=" +keyf
            

    if args[0] == "csr-from-cert":
        lines = []
        if not os.path.exists(wdir):
            print "directory " + wdir + " does not exist"
            usage()
            sys.exit(2)
        if os.path.exists(infile):
            label, lines = read_csv(infile)
        else:
            print infile + " does not exist, will try command line CNs"
        for cf in args[1:]:
            lines.append(cf)
        for cf in lines:
            fqdn, service = CertLib.getFQDNfromCert(cf)
            if not quiet:
                print fqdn + " " + service
            if service:
                reqf = wdir + "/" + fqdn + "-" + service + "-req.pem"
                keyf = wdir + "/" + fqdn + "-" + service + "-key.pem"
            else:
                reqf = wdir + "/" + fqdn + "-req.pem"
                keyf = wdir + "/" + fqdn + "-key.pem"            
                
            print "generating CSR for " + cf
            req = CertLib.genCSRfromCert(reqf,keyf,cf)
            print "wrote CSR=" + reqf + " key=" +keyf


    if args[0] == "csr-submit":
        if not os.path.exists(KEYFILE):
            print "credential missing " + cdir + "/" + KEYFILE
            usage()
            sys.exit(2)
        if not os.path.exists(CERTFILE):
            print "credential missing " + cdir + "/" + CERTFILE
            usage()
            sys.exit(2)

        lines = []
        if not os.path.exists(wdir):
            print "directory " + wdir + " does not exist"
            usage()
            sys.exit(2)
        if len(infile) > 0:
            if os.path.exists(infile):
                label, lines = read_csv(infile)
            else:
                print infile + " does not exist, will try command line CSRs"
        for csr in args[1:]:
            if not quiet:
                print csr
            lines.append(csr)
            
        if post:
            ctx = SSL.Context('sslv3')
            ctx.load_cert(CERTFILE,KEYFILE)
            #ctx.load_verify_locations('amber.pem', '')
            #ctx.set_verify(SSL.verify_peer, 10)
            if ssldebug > 0:
                ctx.set_info_callback()
        for reqf in lines:
            contents = reqPrep(email,RA,VO,'',reqf)
            if not quiet:
                print contents
            if post:
                conn = httpslib.HTTPSConnection(
                    CAS[CA]['addr'],CAS[CA]['port'],ssl_context=ctx)
                conn.set_debuglevel(ssldebug)
                conn.request("POST",CAS[CA]['path'],contents,headers)
                r1 = conn.getresponse()
                if not quiet:
                    print r1.status, r1.reason
                data1 = r1.read()
                cert = get_cert(data1)
                certf = reqf.replace("-req.pem","-cert.pem")
                if not quiet:
                    print certf
                    print data1
                try:
                    cf = open(certf,'w')
                    cf.write(cert)
                    cf.close()
                    print "wrote " + certf
                except:
                    print "Error openning cert file " + certf
                conn.close()

            
    if args[0] == "cn-submit":
        if not os.path.exists(KEYFILE):
            print "credential missing " + cdir + "/" + KEYFILE
            usage()
            sys.exit(2)
        if not os.path.exists(CERTFILE):
            print "credential missing " + cdir + "/" + CERTFILE
            usage()
            sys.exit(2)
            
        for mite in args[1:]:
            miteb = CertLib.cncheck(mite)
            if len(miteb[1]) > 0:
                cnlist.append(miteb)
        if os.path.exists(infile):
            for mite in fileinput.input(infile):
                miteb = chcheck(mite)
                if len(miteb[1]) > 0:
                    cnlist.append(miteb)

        if not quiet:
            for cn in cnlist:
                print cn

        if post:
            ctx = SSL.Context('sslv3')
            ctx.load_cert(CERTFILE,KEYFILE)
            #ctx.load_verify_locations('amber.pem', '')
            #ctx.set_verify(SSL.verify_peer, 10)
            if ssldebug > 0:
                ctx.set_info_callback()

        for cn in cnlist:
            if cn[2]:
                ss = "-" + cn[2]
                sn = cn[2] + "/"
            else:
                ss = ""
                sn = ""
            reqf = wdir + "/" + cn[1] + ss + "-req.pem"
            keyf = wdir + "/" + cn[1] + ss + "-key.pem"
            cns = sn + cn[1]
            req = CertLib.genServiceCSR(reqf,keyf,cn[1],cn[2])
            contents = reqPrep(email,RA,VO,req)

            if not quiet:
                print contents

            if post:
                conn = httpslib.HTTPSConnection(
                    CAS[CA]['addr'],CAS[CA]['port'],ssl_context=ctx)
                conn.set_debuglevel(ssldebug)
                conn.request("POST",CAS[CA]['path'],contents,headers)
                r1 = conn.getresponse()
                print r1.status, r1.reason
                data1 = r1.read()
                print data1
                conn.close()
