package CertLib;
use strict;
use LWP::UserAgent;
use Date::Format;

# $Id: CertLib.pm,v 1.3 2007/03/19 23:57:43 olson Exp $
# $Source: /home/olson/MyCVS/cert-scripts/lib/CertLib.pm,v $
# $Date: 2007/03/19 23:57:43 $
# $Author: olson $

my $pkgversion = "V2-7";
# V2-5a fixes status checking bug for dowloaded URLs

my $ua_timeout = 5;

my $debug = 1;


################################################################################
# stuff for the OSG cert-scripts package in VDT
################################################################################

sub new(;$$) {
    my $class = shift;
    if (@_) {
	$ua_timeout = shift;
	$debug = shift;
    }
    # &checkversion();
    my $self = {};
    $self->{RAS} = load_ras("http://www.ppdg.net/RA/communities/doegrids-ras.txt");
#    print $self->{RAS};
    $self->{OSGVOS} = load_vos("http://oim.grid.iu.edu/pub/vo/show.php?format=plain-text");
#    $self->{OSGVOS} = load_vos("http://www.grid.iu.edu/osg-includes/vos.php");
    
    my %doegridsca = (
		  enrollment => 'https://pki1.doegrids.org/enrollment',
		gridadmin => 'https://pki1.doegrids.org/certbasedenrollment',
		  cpcps => 'http://www.doegrids.org/Docs/CP-CPS.pdf',
		  retrieve =>
    'https://pki1.doegrids.org/displayBySerial?op=displayBySerial&serialNumber=',
		  retreq => 'https://pki1.doegrids.org/ca/processReq?seqNum=');

    my %doegridscaold = (
		      enrollment => 'https://pki1.doegrids.org/enrollment',
	       gridadmin => 'https://pki1.doegrids.org/enrollment',
		      cpcps => 'http://www.doegrids.org/Docs/CP-CPS.pdf',
		      retrieve => 
    'https://pki1.doegrids.org/displayBySerial?op=displayBySerial&serialNumber=',
		     retreq => 'https://pki1.doegrids.org:8100/ca/processReq?seqNum=');
    my %testca = (
		  enrollment => 'https://amber.es.net/enrollment',
	       gridadmin => 'https://pki1.doegrids.org/enrollment',
		  cpcps => 'none',
		  retrieve =>
    'https://amber.es.net/displayBySerial?op=displayBySerial&serialNumber=',
		 retreq => 'https://pki1.doegrids.org:8100/ca/processReq?seqNum=');
#    my %testca2 = (
#		  enrollment => 'https://copper.es.net:1444/enrollment',
#		gridadmin => 'https://copper.es.net:1444/certbasedenrollment',
#		  cpcps => 'none',
#		  retrieve =>
#    'http://copper.es.net:1082/displayBySerial?op=displayBySerial&serialNumber=',
#		retreq => 'https://copper.es.net:8101/ca/processReq?seqNum=',
#		prerevoke => 'https://copper.es.net:1444/revocation',
#		revoke => 'https://copper.es.net:1444/doRevoke');
    my %testca2 = (
		  enrollment => 'https://copper.es.net/enrollment',
		gridadmin => 'https://copper.es.net/certbasedenrollment',
		  cpcps => 'none',
		  retrieve =>
    'https://copper.es.net/displayBySerial?op=displayBySerial&serialNumber=',
		retreq => 'https://copper.es.net:8100/ca/processReq?seqNum=',
		prerevoke => 'https://copper.es.net/revocation',
		revoke => 'https://copper.es.net/doRevoke');
    my %ppdgca = (
		  enrollment => 'https://www.ppdg.net/cgi-bin/ssl/test4567.cgi',
		  gridadmin => 'https://www.ppdg.net/cgi-bin/ssl/test4567.cgi',
		  cpcps => 'none',
		  retrieve =>
		  'https://www.ppdg.net/cgi-bin/ssl/outofluck',
		 retreq => 'https://www.ppdg.net/cgi-bin/ssl/outofluck');
    my %cas = ( doegrids => \%doegridsca,
		testca => \%testca,
		test2 => \%testca2,
		ppdg => \%ppdgca);
    $self->{CA} = \%cas;
    bless ($self, $class);
    return $self;
}

sub osgvos {
    my $self = shift;
    return %{$self->{OSGVOS}};
}

sub getversion {
    return $pkgversion;
}

sub checkversion ($$) {
    my $self = shift;
    my $script = shift;
    my $sversion = shift;
    if ( $pkgversion ne $sversion ) {
	print "CertLib.pm and " . $script . " are not same version\n";
	print "  " . $script . " version " . $sversion . "\n";
    }
    my $ua = LWP::UserAgent->new();
    my $vurl = "http://www.ppdg.net/RA/cert-scripts/cert-scripts-versions.txt";
    $ua->env_proxy();
    $ua->proxy(https => undef);
    $ua->timeout($ua_timeout);
    if ($debug > 0) {
	print "UserAgent timeout $ua_timeout\n";
    }
    $ua->agent("CertLib/" . $pkgversion . " ");
    if ($debug > 0) { print "About to check version\n"; }
    my $req = HTTP::Request->new(GET => $vurl);
    my $res = $ua->request($req);
    if ($debug > 0) { print $res->code() . " - " . $res->content() . "\n"; }
    if ($res->code() == 200) {
	my @avers = split('\n',$res->content);
	my ($label,$descr,$foundit); $foundit = 0;
	while (($label,$descr) = split(":",pop(@avers))) {
#	    print "label $label, descr $descr\n";
	    if ($label eq $pkgversion) {
		print "checking CertLib version, $label, $descr\n";
		$foundit = 1;
	    }
	}
	if ($foundit == 0) {
	    if ($debug > 0) {
		print "did not find version to check\n";
	    }
	}
    } else {
	if ($debug > 0) {
	    print "Error reading versions file $vurl\n";
	}
    }
    return $pkgversion;
}

sub ras {
    my $self = shift;
    return %{$self->{RAS}};
}

sub cas {
    my $self = shift;
    return %{$self->{CA}};
}

sub set_ua_timeout($) {
    my $new_timeout = shift;
    my $old_timeout = $ua_timeout;
    $ua_timeout = $new_timeout;
    return $old_timeout;
}

sub load_ras($) {
    my $rasurl = shift;
    my %ras;
    my %defras = (
	     anl => 'ANL: Argonne National Lab',
	     esg => 'ESG: Earth System Grid',
	     esnet => 'ESnet: DOE Science network',
	     fnal => 'FNAL: Fermilab certificates',
	     fusiongrid => 'FusionGRID: National Fusion Grid Project',
	     lbnl => 'LBNL: Berkeley Lab',
	     lcg => 'LCG: LHC Computing Grid',
	     nersc => 'NERSC: computer center, see www.nersc.gov',
	     ornl => 'ORNL: Oak Ridge National Lab',
	     osg => 'OSG: Open Science Grid (choose this if nothing else applies)',
	     philips => 'Philips: Philips Research U.S.',
	     pnnl => 'PNNL: Pacific Northwest National Lab'
		 );
    my $ua = LWP::UserAgent->new;
    # Use a proxy if user has proxy set in their environment:
    $ua->env_proxy;
    # BUG:  http://rt.cpan.org/Public/Bug/Display.html?id=1894
    #  - Cannot use LWP proxy for https requests, Crypt::SSLeay
    #    will do its own proxy negotiation correctly if the
    #    https_proxy variable is set in the user's environment.
    $ua->proxy(https => undef);
    $ua->timeout($ua_timeout);
    $ua->agent("CertLib/" . $pkgversion . " ");
    my $req = HTTP::Request->new(GET => $rasurl);
    my $res = $ua->request($req);
    my $rc = $res->code;
    if ( $debug ) { print "RAS as string: " . $res->as_string . 
		      "\n   content: " . $res->content .
			"\n   code:  $rc\n"; }
    if ( $rc == 200) {
	my @aras = split('\n',$res->content);
	my ($label,$descr);
	while (($label,$descr) = split(",",pop(@aras))) {
	    $ras{$label} = $descr;
#	    print "label $label, descr $descr\n";
	}
	
    } else {
	%ras = %defras;
    }
    return \%ras;
}

######################### list RAs #########################
sub list_ras($$\%) {
  my $self = shift;
  my $prefix = shift;
  my $sep = shift;
  my $ralistref = shift;
  my $akey;
  for $akey ( sort(keys(%{$ralistref})) ) {
    my ($ra,$radesc) = split(":",$$ralistref{$akey});
    printf("%s%s%s%s\n",$prefix,$ra,$sep,$radesc);
  }
  return;
}

#################### get_osgvolist #####################
sub load_vos($) {
  my $url = shift;
  my %vos;

  my %inst_vos = (
		  BNL => 'Brookhaven lab researchers',
		  JLab => 'Jefferson Lab researchers',
		  SLAC => 'SLAC National Accelerator Laboratory researchers',
		 );
  my $ua = LWP::UserAgent->new;
  # Use a proxy if user has proxy set in their environment:
  $ua->env_proxy;
  # BUG:  http://rt.cpan.org/Public/Bug/Display.html?id=1894
  #  - Cannot use LWP proxy for https requests, Crypt::SSLeay
  #    will do its own proxy negotiation correctly if the
  #    https_proxy variable is set in the user's environment.
  $ua->proxy(https => undef);
  
  $ua->agent("RequestScript/" . $pkgversion . " ");
  my $req = HTTP::Request->new(GET => $url);
  my $res = $ua->request($req);
  if ( $debug ) { print " VOs content: " . $res->content .
		    "\n code: " . $res->code . "\n"; }
  my (@osgvos,%vos,$vo,$desc);
  if ( $res->code == 200 ) {
    @osgvos = split("\n",$res->content);
  } else {
    my $avo;
    while( $avo = <CertLib::DATA> ) {
      if ( $debug ) { print $avo; }
      push(@osgvos,$avo);
    }
  }
  #      my $i;
  #      for ($i=0; $i< @osgvos; $i++) {
  #	#	print $osgvos[$i]."\n";
  #	$vo = $desc = "";
  #	($vo, $desc) = split('\|\|', $osgvos[$i]);
  #	$vo =~ s/^\s*//; # clean left whitespace
  #	$vo =~ s/\s*$//; # clean right whitespace
  #	$desc =~ s/^\s*//; # clean left whitespace
  #	$desc =~ s/\s*$//; # clean right whitespace	
  #	#	printf("vo: %s, desc: %s\n",$vo,$desc);
  foreach my $line (@osgvos){
    next if $line =~ /^#/; # Ignore comments
    next if $line =~ /^\s*$/; # Ignore empty lines
    +my @voline = split(/,/,$line);
    next if (!defined $voline[0] || !defined $voline[1]); #Line not correctly defined line
    $vo = ucfirst $voline[0];
#    $desc = $vo.": ".$voline[1];
    $desc = $voline[1];
#    $vo = lc($vo);
### filter or munge Fermilab VO names here
###
#    my $ucvo = uc $vo;
# check for long Fermilab VO name and hyphenate it
#    my $flloc = index($vo,"fermilab");
#    my $fllen = length($vo);
#  if ( $debug ) {
#    printf("%s %d %d\n",$vo,$flloc,$fllen);
#  }
#  if ($flloc == 0 && $fllen>8) {
#    $vo = "fermilab-" . substr($vo,8);
#  }
    $vos{$vo} = $desc;
  }
  while (($vo,$desc) = each %inst_vos) {
    $vos{$vo} = $desc;
  }
  return \%vos;
}


########################## list VOs #########################
sub list_vos($$\%) {
  my $self = shift;
  my $prefix = shift;
  my $sep = shift;
  my $volistref = shift;
  my $akey;
  for $akey ( sort {lc $a cmp lc $b} keys(%{$volistref}) ) {
    my $vo = $akey;
    my $vodesc = $$volistref{$akey};
    printf("%s%s%s%s\n",$prefix,$vo,$sep,$vodesc);
  }
  return;
}

################## read in parameters ##################
##  recipe taken from http://www.unix.com.ua/orelly/perl/cookbook/ch08_17.htm
#
#  $infile = pathname of file to read (name = value format)
#  $pref = reference to input hash
#  $behavior = flag to indicate behavior for names listed in file
#            = a - means add names in file to keys in $$pref hash
#            = !a - means don't add name as new key, and print warning
#
sub read_params($$$) {
  my $self = shift;
  my $infile = shift;
  my $pref = shift;
  my $behavior = shift;
  if ( ! $behavior ) { $behavior = "n"; }
  my $cnt=0;
  if ( $debug > 0 ) { print "read_params: try reading ", $infile, "\n"; }
  open(PARAMS," < $infile") || return;
  while (<PARAMS>) {
    chomp;                  # no newline
    s/#.*//;                # no comments
    s/^\s+//;               # no leading white
    s/\s+$//;               # no trailing white
    next unless length;     # anything left?
    if ( index($_,"=") > 0 ) {
      my ($var, $value) = split(/\s*=\s*/, $_, 2);
      if ( exists $$pref{$var}  ||  "a" =~ /$behavior/ ) {
	$$pref{$var} = $value;
	if ( $debug > 0 ) { print " var=value: ",$var," = ",$value,"\n";}
      } else {
	print " read_params warning: undefined ",$var," = ",$value,"\n";
      }
      $cnt++;
    }
  }
  close( PARAMS );
  print "read ", $cnt, " parameters from ", $infile, "\n";
  return $cnt;
}

#######################  write parameters to a file ####################
## the converse to read_params
#
sub write_params($$$) {
  my $self = shift;
  my $outfile = shift;
  my $pref = shift;
  my $sref = shift; # reference to array of keys to suppress
  my $cref = shift; # reference to array of comments to include
  my $ncmnt = @$cref;
  if ( $debug > 0 ) { print " writing parameters to file ",$outfile,"\n"; }
  my $cnt=0;
  open (PARAMS, "> $outfile") || return $cnt;
  print PARAMS "## writing ", $outfile," at ",ctime(time);
  for my $cmnt ( @$cref ) {
    print PARAMS "# ",$cmnt,"\n";
  }
  foreach my $k ( keys( %$pref ) ) {
    if ( $$pref{$k} ) {
      my $c="";
      for my $w ( @$sref ) {
	if ( $k =~ /$w/ ) { $c = "# "; last; }
      }
      print PARAMS $c,$k," = ",$$pref{$k},"\n";
      $cnt++;
      if ( $debug > 0 ) { print $k," = ",$$pref{$k},"\n"; }
    }
  }
  return $cnt;
}
END {}

1;
#
# below is list of VOs from the OIM url
__DATA__
## vo_short_name,vo_long_name,primary_url,aup_url,voms_url,support_url,sc_short_name,active
ATLAS,United States ATLAS Collaboration,http://www.usatlas.bnl.gov,http://grid.racf.bnl.gov/GUMS/guide_howto_configuration.html,http://grid.racf.bnl.gov/GUMS/guide_howto_configuration.html, Support URL missing!,USATLAS,1
CDF,Collider Detector at Fermilab,http://www-cdf.fnal.gov,http://www-cdf.fnal.gov,https://voms.cnaf.infn.it:8443/voms/cdf/,http://cdfcaf.fnal.gov,Fermilab,1
CIGI,CyberInfrastructure and Geospatial Information Laboratory,http://www.cigi.uiuc.edu,http://www.cigi.uiuc.edu,http://www.cigi.uiuc.edu,http://www.cigi.uiuc.edu,CIGI,1
CMS,Compact Muon Solenoid,http://www.uscms.org,http://www.uscms.org/SoftwareComputing/Grid/Config/,https://voms.fnal.gov:8443/edg-voms-admin/uscms/index.html,http://helpdesk.fnal.gov,USCMS,1
CompBioGrid,CompBioGrid,http://www.compbiogrid.org/,http://www.compbiogrid.org/,https://voms.compbiogrid.org:8443/voms/CompBioGrid/,http://www.compbiogrid.org/support,UCHC,1
DES,Dark Energy Survey,https://www.darkenergysurvey.org/,https://www.darkenergysurvey.org/,https://voms.fnal.gov:8443/edg-voms-admin/des/services/VOMSAdmin, Support URL missing!,Fermilab,1
DOSAR,Distributed Organization for Scientific and Academic Research,http://www-hep.uta.edu/dosar/,https://thegrid.dpcc.uta.edu:8443/edg-voms-admin/dosar/grid.html,https://thegrid.dpcc.uta.edu:8443/edg-voms-admin/dosar/grid.html,http://www-hep.uta.edu/dosar/ ,DOSAR,1
DZero,D0 Experiment at Fermilab,http://www-d0.fnal.gov/,http://www-d0.fnal.gov/,vomss://voms.fnal.gov:8443/edg-voms-admin/dzero?/dzero/users,http://www-d0.fnal.gov/VO/DZero_VO_Instructions.html,Fermilab,1
Engage,Engagement,https://twiki.grid.iu.edu/bin/view/Engagement/WebHome,https://osg-engage.renci.org:8443/vomrs/Engage/vomrs,https://osg-engage.renci.org:8443/vomrs/Engage/vomrs,https://twiki.grid.iu.edu/bin/view/Engagement/WebHome,Engagement,1
Fermilab,Fermi National Accelerator Center,http://fermigrid.fnal.gov/,http://fermigrid.fnal.gov/,https://vomrs.fnal.gov:8443/vomrs/vo-fermilab/vomrs,http://fermigrid.fnal.gov/,Fermilab,1
FermilabAccelerator,Fermilab/Accelerator,http://www-bd.fnal.gov/,http://fermigrid.fnal.gov/,https://vomrs.fnal.gov:8443/vomrs/vo-fermilab/vomrs,http://fermigrid.fnal.gov/,Fermilab,1
FermilabAstro,Fermilab/Astro,http://www.fnal.gov/faw/experimentsprojects/index.html,http://fermigrid.fnal.gov/,https://vomrs.fnal.gov:8443/vomrs/vo-fermilab/vomrs,http://fermigrid.fnal.gov/,Fermilab,1
FermilabCdms,Fermilab/Cdms,http://ppd.fnal.gov/experiments/cdms/,http://fermigrid.fnal.gov/,https://vomrs.fnal.gov:8443/vomrs/vo-fermilab/vomrs,http://fermigrid.fnal.gov/,Fermilab,1
FermilabGrid,fermilab VO grid group,http://fermigrid.fnal.gov/,http://fermigrid.fnal.gov/,https://vomrs.fnal.gov:8443/vomrs/vo-fermilab/vomrs,http://fermigrid.fnal.gov/,Fermilab,1
FermilabHypercp,Fermilab/Hypercp,http://ppd.fnal.gov/experiments/e871/welcome.html,http://fermigrid.fnal.gov/,https://vomrs.fnal.gov:8443/vomrs/vo-fermilab/vomrs,http://fermigrid.fnal.gov/,Fermilab,1
FermilabKTeV,Fermilab/KTeV,http://ktev.fnal.gov/public/,http://fermigrid.fnal.gov/,https://vomrs.fnal.gov:8443/vomrs/vo-fermilab/vomrs,http://fermigrid.fnal.gov/,Fermilab,1
FermilabMinerva,Fermilab/Minerva,http://www.pas.rochester.edu/minerva/,http://fermigrid.fnal.gov/,https://vomrs.fnal.gov:8443/vomrs/vo-fermilab/vomrs,http://fermigrid.fnal.gov/,Fermilab,1
FermilabMiniboone,Fermilab/Miniboone,http://www-boone.fnal.gov/,http://fermigrid.fnal.gov/,https://vomrs.fnal.gov:8443/vomrs/vo-fermilab/vomrs,http://fermigrid.fnal.gov/,Fermilab,1
FermilabMinos,Fermilab/Minos,http://www-numi.fnal.gov/,http://fermigrid.fnal.gov/,https://vomrs.fnal.gov:8443/vomrs/vo-fermilab/vomrs,http://fermigrid.fnal.gov/,Fermilab,1
FermilabMipp,Fermilab/Mipp,http://ppd.fnal.gov/experiments/e907/,http://fermigrid.fnal.gov/,https://vomrs.fnal.gov:8443/vomrs/vo-fermilab/vomrs,http://fermigrid.fnal.gov/,Fermilab,1
FermilabNova,Fermilab/Nova,http://www-nova.fnal.gov/,http://fermigrid.fnal.gov/,https://vomrs.fnal.gov:8443/vomrs/vo-fermilab/vomrs,http://fermigrid.fnal.gov/,Fermilab,1
FermilabNumi,Fermilab/Numi,http://fermigrid.fnal.gov/,http://fermigrid.fnal.gov/,https://vomrs.fnal.gov:8443/vomrs/vo-fermilab/vomrs,http://fermigrid.fnal.gov/,Fermilab,1
FermilabPatriot,Fermilab/Patriot,http://theory.fnal.gov/,http://fermigrid.fnal.gov/,https://vomrs.fnal.gov:8443/vomrs/vo-fermilab/vomrs,http://fermigrid.fnal.gov/,Fermilab,1
FermilabTest,Fermilab/Test,http://fermigrid.fnal.gov/,http://fermigrid.fnal.gov/,https://vomrs.fnal.gov:8443/vomrs/vo-fermilab/vomrs,http://fermigrid.fnal.gov/,Fermilab,1
FermilabTheory,Fermilab/Theory,http://theory.fnal.gov/,http://fermigrid.fnal.gov/,https://vomrs.fnal.gov:8443/vomrs/vo-fermilab/vomrs,http://fermigrid.fnal.gov/,Fermilab,1
geant4,Geant4 Software Toolkit,http://www.cern.ch/geant4,http://www.cern.ch/geant4,https://lcg-voms.cern.ch:8443/vo/geant4/vomrs, Support URL missing!,CERN,1
GLOW,Grid Laboratory of Wisconsin,http://www.cs.wisc.edu/condor/glow/,http://www.cs.wisc.edu/condor/glow/,https://glow-voms.cs.wisc.edu:8443/edg-voms-admin/GLOW/index.html,http://glow.cs.wisc.edu/,GLOW-TECH,1
GPN,Great Plains Network,http://www.greatplains.net/,http://t2.unl.edu/documentation/gpn,https://t2.unl.edu:8443/voms/gpn/,http://t2.unl.edu/documentation/gpn,GPN,1
GRASE,Group Researching Advances in Software Engineering at University of New York at Buffalo,http://osg.ccr.buffalo.edu/grase/,http://osg.ccr.buffalo.edu/grase/,https://dylan.ccr.buffalo.edu:8443/edg-voms-admin/GRASE/,http://osg.ccr.buffalo.edu/contact/grid_help.php,GRASE,1
GROW,Grid Research and Education Group at Iowa,http://www.its.uiowa.edu/research/grow,http://www.its.uiowa.edu/research/grow,https://grow-voms.its.uiowa.edu:8443/voms/GROW,http://www.its.uiowa.edu/research/grow/osg/,GROW-GOC,1
GUGrid,Georgetown University Grid,http://gugrid.arc.georgetown.edu,http://gugrid.arc.georgetown.edu,https://gugrid.arc.georgetown.edu:8443/voms/GUGrid/,http://gugrid.arc.georgetown.edu,GUGrid,1
i2u2,Interactions in Understanding the Universe Initiative,https://fermigrid2.fnal.gov:8443/vomrs/i2u2/vomrs,https://voms.fnal.gov:8443/voms/i2u2/services/VOMSAdmin,https://voms.fnal.gov:8443/voms/i2u2/services/VOMSAdmin,http://quarknet.fnal.gov/e-labs,Fermilab,1
IceCube,IceCube Neutrino Telescope,https://grid-voms.desy.de:8443/voms/icecube/,http://www.icecube.wisc.edu/collaboration/aup.php,http://www.icecube.wisc.edu/collaboration/aup.php,,I3_support,1
ILC,International Linear Collider,http://ilc.fnal.gov/,http://fermigrid.fnal.gov/,https://voms.fnal.gov:8443/voms/ilc,http://computing.fnal.gov/helpdesk/,Fermilab,1
LIGO,Laser Interferometer Gravitational-Wave Observatory,http://www.ligo.org,http://voms.phys.uwm.edu/lams/account.shtml,https://voms.ligo.org:8443/voms/LIGO/Configuration.do,http://www.lsc-group.phys.uwm.edu/lscdatagrid/OSG/index.html,LIGO,1
mariachi,Mixed Apparatus for Radar Investigation of Cosmic-rays of  High Ionization Experiment,http://www-mariachi.physics.sunysb.edu/wiki/,https://www-mariachi.physics.sunysb.edu/voms/mariachi/,https://www-mariachi.physics.sunysb.edu/voms/mariachi/services/VOMSAdmin,http://www-mariachi.physics.sunysb.edu/wiki/index.php/VOSupport,Mariachi,1
MIS,OSG Monitoring Information System,http://www.ivdgl.org/ivdgl/mis-vo-privilege.html,http://www.ivdgl.org/ivdgl/mis-vo-privilege.html,https://grid03.uits.indiana.edu:8443/edg-voms-admin/vos/webui/UserRequest, Support URL missing!,OSG-GOC,1
nanoHUB,nanoHUB Network for Computational Nanotechnology (NCN),http://www.nanohub.org,http://www.nanohub.org,https://fermigrid2.fnal.gov:8443/edg-voms-admin/nanohub/index.html,http://www.nanohub.org/support,nanoHUB,1
NWICG,Northwest Indiana Computational Grid,http://www.nwicgrid.org,http://www.nwicgrid.org,https://vo.rcac.purdue.edu:8443/voms/NWICG/services/VOMSAdmin, Support URL missing!,NWICG,1
NYSGRID,NYSGRID,http://www.nysgrid.org/,https://dylan.ccr.buffalo.edu:8443/edg-voms-admin/NYSGRID,https://dylan.ccr.buffalo.edu:8443/edg-voms-admin/NYSGRID, Support URL missing!,GRASE,1
Ops,WLCG Operations Group,http://www.grid.iu.edu,http://www.ivdgl.org/ivdgl/ops-vo-privilege.html,https://lcg-voms.cern.ch:8443/vo/ops/vomrs?path=/RootNode&action=execute, Support URL missing!,OSG-GOC,1
OSG,Open Science Grid,http://www.opensciencegrid.org,http://osg-docdb.opensciencegrid.org/0007/000742/003/OSG%20VO%20AUP.pdf,https://voms.opensciencegrid.org:8443/vomrs/osg/vomrs, Support URL missing!,CSC,1
OSGEDU,OSG Education Activity,http://www.opensciencegrid.org/Education,http://www.ivdgl.org/ivdgl/osgedu-vo-privilege,https://grid03.uits.indiana.edu:8443/edg-voms-admin/vos/webui/UserRequest, Support URL missing!,CSC,1
SBGrid,Structural Biology Grid,http://www.sbgrid.org,http://www.sbgrid.org/contact.php,https://www.sbgrid.org:8443/,http://www.sbgrid.org/contact.php,CSC,1
SDSS,Sloan Digital Sky Survey,http://www.sdss.org,http://www.sdss.org,https://voms.fnal.gov:8443/voms/sdss/index.html?/sdss/production/, Support URL missing!,Fermilab,1
STAR,Solenoidal Tracker at RHIC,http://www.star.bnl.gov,http://www.star.bnl.gov,vomss://vo.racf.bnl.gov:8443/edg-voms-admin/star, Support URL missing!,STAR,1
