#!/usr/bin/env python
#
# a python equivalent to the perl CertLib.pm
#
#package CertLib;
#use strict;
#use LWP::UserAgent;

import urllib
import socket
from M2Crypto import RSA, X509, EVP, m2, Rand, Err
#from M2Crypto import RSA, EVP, m2, Rand, Err
#import X509
import sys, os
from stat import *

#global pkgversion, debug, RAS, OSGVOS, CA

pkgversion = "V2-7"
ua_timeout = 10
debug = 0

def getversion():
    global pkgversion
    return pkgversion

def checkversion(script,sversion):
    global debug, pkgversion
    vurl = "http://www.ppdg.net/RA/cert-scripts/cert-scripts-versions.txt"
    if not pkgversion in sversion:
        print "CertLib.py and " + script + " are not same version"
        print "  " + script + " version " + sversion
    try:
        f = urllib.urlopen(vurl)
#        print "read f"
        page = f.read(10000)
#        print "got page " + page
        lines = page.split("\n")
        foundit = 0
        for aline in lines:
#            print "line " + aline
            if len(aline) > 0:
                if '#' in aline[0]:
                    continue
            words = aline.split(":")
            if len(words) > 1:
                if pkgversion in words[0]:
#                    print "found it"
                    print "checking CertLib version, " + words[0] + ", " + words[1]
                    foundit = 1
    except:
        if debug:
            print "did not read " + vurl
            print sys.exc_info()[0]
    return pkgversion

def set_ua_timeout(t):
    global ua_timeout
    oldt = ua_timeout
    ua_timeout = t
    socket.setdefaulttimeout(ua_timeout)
    return oldt

def generateRSAKey():
    return RSA.gen_key(2048, m2.RSA_F4)

def makePKey(key):
    pkey = EVP.PKey()
    pkey.assign_rsa(key)
    return pkey
    
def makeRequest(pkey,host,service=''):
    global debug
    req = X509.Request()
    # Seems to default to 0, but we can now set it as well, so just API test
    req.set_version(req.get_version())
    req.set_pubkey(pkey)
    # req.set_pubkey(EVP.PKey(req.get_pubkey()))# Just a test of the API
    name = X509.X509_Name()
    name.DC = 'org'
    name.DC = 'doegrids'
    name.OU = 'Services'
    if service :
        service = service + '/'
    name.CN = service + host
    req.set_subject_name(name)
    # ext1 = X509.new_extension('subjectAltName', 'DNS:foobar.example.com')
    # ext2 = X509.new_extension('nsComment', 'Hello there')
    # extstack = X509.X509_Extension_Stack()
    # extstack.push(ext1)
    # extstack.push(ext2)

    # assert(extstack[1].get_name() == 'nsComment')
    
    # req.add_extensions(extstack)
    req.sign(pkey, 'sha1')
    return req

def genServiceCSR(reqfile,keyfile,host,service='',encrypt=''):
    """
    generate a PKCS10 certificate signing request

    @type reqfile: string
    @param reqfile: name of file to save CSR in PEM format

    @type keyfile: string
    @param keyfile: name of file to save private key in PEM format

    @type host: string
    @param host: FQDN of host in the subject of the certificate

    @type service: string
    @param service: optional name of service on the specified host

    @type encrypt: string
    @param encrypt: any value will cause private key to be encrypted.
    There is an interactive prompt for the passphrase for encryption.
    If you omit the argument or pass a null string the private key is
    stored in the clear.

    Subject names are of the form
    CN=[<service>/]host,OU=Services,DC=doegrids,DC=org
    """
    rsa = generateRSAKey()
    pkey = makePKey(rsa)
    req = makeRequest(pkey,host,service)
    req.save_pem(reqfile)
    if encrypt :
        rsa.save_key(keyfile,'aes_256_cbc')
        os.chmod(keyfile,S_IRUSR+S_IWUSR)
    else :
        rsa.save_key(keyfile,None)
        os.chmod(keyfile,S_IRUSR+S_IWUSR)
    return req

def cncheck(icn):
    """
    checks a string if it seems valid for a CN and returns
    CN, host, service

    @type icn: string
    @param icn: possible CN string
    """

    #strip any leading '*='
    ieq = icn.find("=")
    if ieq > -1:
        icn = icn[ieq+1:]
    islash = icn.find("/")
    if islash > -1:
        service = icn[0:islash]
        fqdn = icn[islash+1:]
    else:
        service = ""
        fqdn = icn
    dots = fqdn.count(".")
    if dots == 0:
        print "W no . in " + fqdn
    return icn, fqdn, service

def getFQDNfromCert(certfile):
    """
    get FQDN and service elements of a service certificate CN

    @type certfile: string
    @param certfile: path name to certificate file
    """
    cert = X509.load_cert(certfile)
    subject = cert.get_subject()
    ans = subject.as_text()
    cn = ans[ans.find("CN=")+3:]
    # print ans + " CN " + cn
    ocn, fqdn, service = cncheck(cn)
    return fqdn, service
    
def genCSRfromCert(reqfile,keyfile,certfile,encrypt=''):
    """
    generate a CSR from a certificate file

    @type reqfile: string
    @param reqfile: path name to CSR file in PEM format

    @type keyfile: string
    @param keyfile: path name to private key file
    
    @type certfile: string
    @param certfile: path name to public certificate in PEM format

    @type encrypt: string
    @param encrypt: any value will cause private key to be encrypted.
    There is an interactive prompt for the passphrase for encryption.
    If you omit the argument or pass a null string the private key is
    stored in the clear.
    """
    cert = X509.load_cert(certfile)
    subject = cert.get_subject()
    # print subject.as_text()
    rsa = generateRSAKey()
    pkey = makePKey(rsa)
    req = X509.Request()
    req.set_version(req.get_version())
    req.set_pubkey(pkey)
    req.set_subject_name(subject)
    req.sign(pkey, 'sha1')
    req.save_pem(reqfile)
    if encrypt :
        rsa.save_key(keyfile,'aes_256_cbc')
        os.chmod(keyfile,S_IRUSR+S_IWUSR)
    else :
        rsa.save_key(keyfile,None)
        os.chmod(keyfile,S_IRUSR+S_IWUSR)
    return req    
    

def get_osgvos():
    global OSGVOS
    return OSGVOS

def get_ras():
    global RAS
    return RAS

def get_cas():
    global CA
    return CA

def load_ras(rasurl):
    global debug
    defras = {
        'anl' : 'ANL: Argonne National Lab',
        'esg' : 'ESG: Earth System Grid',
        'esnet' : 'ESnet: DOE Science network',
        'fnal' : 'FNAL: Fermilab certificates',
        'fusiongrid' : 'FusionGRID: National Fusion Grid Project',
        'lbnl' : 'LBNL: Berkeley Lab',
        'lcg' : 'LCG: LHC Computing Grid',
        'nersc' : 'NERSC: computer center, see www.nersc.gov',
        'ornl' : 'ORNL: Oak Ridge National Lab',
        'osg' : 'OSG: Open Science Grid (choose this if nothing else applies)',
        'philips' : 'Philips: Philips Research U.S.',
        'pnnl' : 'PNNL: Pacific Northwest National Lab'
	}
    ras = {}
    try:
        f = urllib.urlopen(rasurl)
        page = f.read(10000)
        lines = page.split("\n")
        for aline in lines:
            words = aline.split(",")
            if len(words) > 1:
                label = words[0]
                descr = words[1]
                ras[label] = descr
    except:
        if debug:
            print sys.exc_info()[0]
        print "CertLib.py - could not read current RA list, will use default"
        ras = defras
    return ras

#################### get_osgvolist #####################
def load_vos(url):
    global debug
    vos = {}
    #
    # make the list below from the plain text for from OIM as
    #  awk -F, '{printf("\t\047%s\047 : \047%s\047,\n",$1,$2)}' vos.txt

    defvos = {
        'ALICE' : 'ALICE collaboration',
        'ATLAS' : 'United States ATLAS Collaboration',
        'CDF' : 'Collider Detector at Fermilab',
        'CIGI' : 'CyberInfrastructure and Geospatial Information Laboratory',
        'CMS' : 'Compact Muon Solenoid',
        'CompBioGrid' : 'CompBioGrid',
        'DES' : 'Dark Energy Survey',
        'DOSAR' : 'Distributed Organization for Scientific and Academic Research',
        'DZero' : 'D0 Experiment at Fermilab',
        'Engage' : 'Engagement',
        'Fermilab' : 'Fermi National Accelerator Laboratory',
        'FermilabAccelerator' : 'Fermilab/Accelerator',
        'FermilabAstro' : 'Fermilab/Astro',
        'FermilabCdms' : 'Fermilab/Cdms',
        'FermilabGrid' : 'fermilab VO grid group',
        'FermilabHypercp' : 'Fermilab/Hypercp',
        'FermilabKTeV' : 'Fermilab/KTeV',
        'FermilabMinerva' : 'Fermilab/Minerva',
        'FermilabMiniboone' : 'Fermilab/Miniboone',
        'FermilabMinos' : 'Fermilab/Minos',
        'FermilabMipp' : 'Fermilab/Mipp',
        'FermilabMu2e' : 'Fermilab/Mu2e',
        'FermilabNova' : 'Fermilab/Nova',
        'FermilabNumi' : 'Fermilab/Numi',
        'FermilabPatriot' : 'Fermilab/Patriot',
        'FermilabTest' : 'Fermilab/Test',
        'FermilabTheory' : 'Fermilab/Theory',
        'geant4' : 'Geant4 Software Toolkit',
        'GLOW' : 'Grid Laboratory of Wisconsin',
        'GPN' : 'Great Plains Network',
        'GRASE' : 'Group Researching Advances in Software Engineering at University of New York at Buffalo',
        'GROW' : 'Grid Research and Education Group at Iowa',
        'GUGrid' : 'Georgetown University Grid',
        'i2u2' : 'Interactions in Understanding the Universe Initiative',
        'IceCube' : 'IceCube Neutrino Telescope',
        'ILC' : 'International Linear Collider',
        'JDEM' : 'Joint Dark Energy Mission',
        'LIGO' : 'Laser Interferometer Gravitational-Wave Observatory',
        'mariachi' : 'Mixed Apparatus for Radar Investigation of Cosmic-rays of  High Ionization Experiment',
        'MIS' : 'OSG Monitoring Information System',
        'nanoHUB' : 'nanoHUB Network for Computational Nanotechnology (NCN)',
        'NWICG' : 'Northwest Indiana Computational Grid',
        'NYSGRID' : 'NYSGRID',
        'Ops' : 'WLCG Operations Group',
        'OSG' : 'Open Science Grid',
        'OSGEDU' : 'OSG Education Activity',
        'SBGrid' : 'Structural Biology Grid',
        'SDSS' : 'Sloan Digital Sky Survey',
        'STAR' : 'Solenoidal Tracker at RHIC'
	}
    inst_vos = {
        'BNL' : 'Brookhaven lab researchers',
        'JLab' : 'Jefferson Lab researchers',
        'SLAC' : 'SLAC National Accelerator Laboratory researchers'
	}
    vos = inst_vos
    try:
        f = urllib.urlopen(url)
        page = f.read(10000)
        osgvos = page.split("\n")
        for avo in osgvos:
            vd = avo.split(",")
            if len(vd) > 1:
                vo = vd[0]
                descr = vd[1]
                vo = vo.strip()
                descr = descr.strip()
                vos[vo] = descr
    except:
        # print "more debug ", debug
        if debug == 1 :
            print "Unexpected error:", sys.exc_info()[0]
        print "CertLib.py - could not read list of VOs, will use default"
        vos = defvos
        vos.update(inst_vos)
        
    if not vos['osg']:
	vos = defvos
        vos.update(inst_vos)

    return vos

# initialize

def init(sdebug):
    global debug, RAS, OSGVOS, CA
    debug = sdebug
    # print "debug ", debug
    
    RAS = load_ras("http://www.ppdg.net/RA/communities/doegrids-ras.txt")

    # OSGVOS = load_vos("http://www.grid.iu.edu/osg-includes/vos.php")
    OSGVOS = load_vos("http://oim.grid.iu.edu/pub/vo/show.php?format=plain-text")


    doegridsca = {
        'addr' : 'pki1.doegrids.org',
        'port' : 443,
        'path' : '/enrollment',
        'cbpath' : '/certbasedenrollment',
        'cpcps' : 'http://www.doegrids.org/Docs/CP-CPS.pdf',
        'retrieve' : 'https://pki1.doegrids.org/displayBySerial?op=displayBySerial&serialNumber=',
        'retreq' : 'https://pki1.doegrids.org/ca/processReq?seqNum='
        }
    
    doegridscaold = {
        'addr' : 'pki1.doegrids.org',
        'port' : 443,
        'path' : '/enrollment',
        'cbpath' : '/enrollment',
        'cpcps' : 'http://www.doegrids.org/Docs/CP-CPS.pdf',
        'retrieve' : 'https://pki1.doegrids.org/displayBySerial?op=displayBySerial&serialNumber='
        }
    testca = {
        'addr' : 'amber.es.net',
        'port' : 443,
        'path' : '/enrollment',
        'cbpath' : '/enrollment',
        'cpcps' : 'none',
        'retrieve' : 'https://amber.es.net/displayBySerial?op=displayBySerial&serialNumber='
        }
#    testca2 = {
#        'addr' : 'copper.es.net',
#        'port' : 1444,
#        'path' : '/enrollment',
#        'cbpath' : '/certbasedenrollment',
#        'cpcps' : 'none',
#        'retrieve' : 'http://copper.es.net:1082/displayBySerial?op=displayBySerial&serialNumber='
#        }
    testca2 = {
        'addr' : 'copper.es.net',
        'port' : 443,
        'path' : '/enrollment',
        'cbpath' : '/certbasedenrollment',
        'cpcps' : 'none',
        'retrieve' : 'http://copper.es.net:443/displayBySerial?op=displayBySerial&serialNumber='
        }
    ppdgca = {
        'addr' : 'www.ppdg.net',
        'port' : 443,
        'path' : '/cgi-bin/ssl/test4567.cgi',
        'cbpath' : '/cgi-bin/ssl/test4567.cgi',
        'cpcps' : 'none',
        'retrieve' : 'https://www.ppdg.net/cgi-bin/ssl/outofluck'
        }
    CA = { 'doegrids' : doegridsca,
           'testca' : testca,
           'test2' : testca2,
           'ppdg' : ppdgca }

    return
