#!/usr/bin/env perl
#
# test the cert-scripts package

use strict;

use Getopt::Long;

use Test::Simple tests => 21;

my $pkgversion="V2-7";

my %SCRIPTS = ( checktime => 'cert-check-time',
		lookup => 'cert-lookup',
		gridadmin => 'cert-gridadmin',
		renew => 'cert-renew',
		request => 'cert-request',
		retrieve => 'cert-retrieve',
		multiga => 'multi-cert-gridadmin'
	      );
my @skeys = keys %SCRIPTS;

#my $argv = join(" ",@ARGV);
#print $argv . "\n";
#print $#ARGV . "\n";

# get commandline options
my @keys = qw(h help test l list verbose noclean ca);
my (%args,@tests);
@args{@keys} = ("") x @keys;
GetOptions( \%args,"h","help","test=s" => \@tests, "l", "list","verbose",
	  "noclean",'ca=s');

#print "after GetOptions\n";
#$argv = join(" ",@ARGV);
#print $argv . "\n";
#print $#ARGV . "\n";

my %testcerts = ( doegrids => '4239',
		  testca => '332',
		  test2 => '23');
my %creddir = ( doegrids => 'ca-pki1',
		testca => 'ca-test1',
		test2 => 'ca-test2');

my $CA = "test2";
if ( ! $args{ca} ) { $args{ca} = $CA; }

if ($args{h} || $args{help}) {
    &usage();
    exit(1);
}

if ($args{ca}) {
    if ($testcerts{$args{ca}}) {
	$CA = $args{ca};
	print "Using CA $CA\n";
    } else {
	print "Invalid CA\n";
	usage();
	exit(1);
    }
}

# $ENV{HTTPS_CA_FILE} = $creddir{$args{ca}} . "/cacertchain.pem";

if ($args{l} || $args{list}) {
    print "available test sets (--test), named after the corresponding script\n";
    foreach my $k (@skeys) {
	printf(" %15s : tests %s\n",$k,$SCRIPTS{$k});
    }
    printf(" %15s : tests %s\n","all","all the scripts, same as omitting --test");
    exit(1);
}

if (@tests == 0 || $tests[0] eq "all" ) {
    @tests = @skeys;  # run all tests
}

# test 1
ok ( $pkgversion eq "V2-7", 'correct version of test.pl' );

#test 2
if ($> < 500) {
    ok (0, 'Tests must be run as non-privileged user.');
    exit(1);
} else {
    ok (1, 'EUID is OK');
}

my ($tok,$tdir,$cmd,$result,@rwords);

for my $test (@tests) {
    if ($args{verbose}) {
	print "running tests for $test\n";
    }
    if ($test eq "lookup") {
	###########################  lookup ou=People
	$tok = 1;
	$tdir = "tmp-lookupp";
	if (! -d $tdir) {
	    $tok = mkdir($tdir);
	}
	if (! $tok) {
	    ok ( $tok, "Cannot create directory $tdir" );
	    next;
	}
	# check for version ##############################
	$cmd = "../bin/cert-lookup -V";
	if ($args{verbose}) { print "Command: $cmd\n"; }
	$result = `$cmd`;
	if ($args{verbose}) { print "Result: $result\n"; }
	if ( &count_match($result,$pkgversion) == 1 ) {
	    $tok = 1;
	} else {
	    $tok = 0;
	}
	ok ($tok, "lookup version");	
	#  do lookup test ###############################
	chdir($tdir);
	$cmd = '../../bin/cert-lookup -p -n "*olson*"';
	if ($args{noclean}) {
	    $cmd = $cmd . " -c";
	}
	if ($args{verbose}) { print "Command: $cmd\n"; }
	$result = `$cmd`;
	if ($args{verbose}) { print "Result: $result\n"; }
	my @rlines = split('\n',$result);
	my $nl = @rlines;
	if ( $args{verbose} ) { print " # lines: $nl\n"; }
	my $rline;
	$tok = 0;
	LPLOOP: foreach $rline (@rlines) {
	  my @rwords = split(' ',$rline);
	  if ($rwords[0] eq "Found" && $rwords[1] > 0) {
	    if ( $args{verbose} ) { print $rline . "\n"; }
	    $tok = 1;
	    last LPLOOP;
	  }
	}
	ok ($tok, "lookup ou=people");
	chdir("..");
	if (! $args{noclean}) {
	    `rm -fr $tdir`;
	}
	####################### lookup ou=Services
	$tok = 1;
	$tdir = "tmp-lookups";
	if (! -d $tdir) {
	    $tok = mkdir($tdir);
	}
	if (! $tok) {
	    ok ( $tok, "Cannot create directory $tdir" );
	    next;
	}
	chdir($tdir);
	$cmd = '../../bin/cert-lookup -s -n "*lbl.gov*"';
	if ($args{noclean}) {
	    $cmd = $cmd . " -c";
	}
	if ($args{verbose}) { print "Command: $cmd\n"; }
	$result = `$cmd`;
	if ($args{verbose}) { print "Result: $result\n"; }
	@rwords = split(' ',$result);
	if ($rwords[0] eq "Found" && $rwords[1] > 0) {
	    ok ($tok, "lookup ou=services");
	} else {
	    $tok = 0;
	    ok ($tok, "lookup ou=services");
	}
	chdir("..");
	if (! $args{noclean}) {
	    `rm -fr $tdir`;
	}
    } elsif ($test eq "checktime") {
	######################### checktime, use ~/.globus/usercert.pem
	$tok = 1;
	$tdir = "tmp-checktime";
	if (! -d $tdir) {
	    $tok = mkdir($tdir);
	}
	if (! $tok) {
	    ok ( $tok, "Cannot create directory $tdir" );
	    next;
	}
	# check for version ##############################
	$cmd = "../bin/cert-check-time -V";
	if ($args{verbose}) { print "Command: $cmd\n"; }
	$result = `$cmd`;
	if ($args{verbose}) { print "Result: $result\n"; }
	if ( &count_match($result,$pkgversion) == 1 ) {
	    $tok = 1;
	} else {
	    $tok = 0;
	}
	ok ($tok, "checktime version");	
	#  do checktime test ###############################
	if (-e "$ENV{HOME}/.globus/usercert.pem") {
	    open(LF, ">$tdir/local-certs.txt");
	    print LF "$ENV{HOME}/.globus/usercert.pem";
	    close(LF);
	}
	$cmd = "../bin/cert-check-time -l -L $tdir/local-certs.txt -d $tdir";
	if ($args{noclean}) {
	    $cmd = $cmd . " -k";
	}
	if ($args{verbose}) {
	    $cmd = $cmd . " -v";
	}
	if ($args{verbose}) { print "Command: $cmd\n"; }
	$result = `$cmd`;
	if ($args{verbose}) { print "Result: $result\n"; }
	if (index($result,"subject=") > 0) {
	    $tok = 1;
	} else {
	    $tok = 0;
	}
	ok ($tok, "checktime");
	if (! $args{noclean}) { `rm -fr $tdir`; }
    } elsif ($test eq "request") {
	######################### request, use OU=Services to no pw
	$tok = 1;
	$tdir = "tmp-request"; 
	if (! -d $tdir) {
	    $tok = mkdir($tdir);
	}
	if (! $tok) {
	    ok ( $tok, "Cannot create directory $tdir" );
	    next;
	}
	# check for version ##############################
	$cmd = "../bin/cert-request -V";
	if ($args{verbose}) { print "Command: $cmd\n"; }
	$result = `$cmd`;
	if ($args{verbose}) { print "Result: $result\n"; }
	if ( count_match($result,$pkgversion) == 3) {
	    $tok = 1;
	} else {
	    $tok = 0;
	}
	ok ($tok, "request version");	
	# do request submission test ######################
	$cmd = "../bin/cert-request --ca $CA --dir $tdir --agree --label test ";
	$cmd = $cmd . " --vo osg --affiliation osg --reason testing --ou s ";
	$cmd = $cmd . " --host host.domain --service testing ";
	$cmd = $cmd . " --email dlolson\@lbl.gov --name 'Admin Name' ";
	$cmd = $cmd . " --phone 123456789 --sponsor_name 'Sponsors Name' ";
	$cmd = $cmd . " --sponsor_phone 987654321 --sponsor_email a\@bb.cc ";
	$cmd = $cmd . " --write $tdir/creq.cfg";
	if ($args{noclean}) {
	    $cmd = $cmd . " --noclean ";
	}
	if ($args{verbose}) {
	    $cmd = $cmd . " --debug ";
	}
	if ($args{verbose}) { print "Command: $cmd\n"; }
	$result = `$cmd`;
	if ($args{verbose}) { print "Result: $result\n"; }
	if (index($result,"Request has been successfully") > 0) {
	    $tok = 1;
	} else {
	    $tok = 0;
	}
	if ( $tok == 1 ) { # try again using saved parameter file
	    $cmd = "../bin/cert-request --agree --read $tdir/creq.cfg";
	    if ($args{noclean}) {
		$cmd = $cmd . " --noclean ";
	    }
	    if ($args{verbose}) {
		$cmd = $cmd . " --debug ";
	    }
	    if ($args{verbose}) { print "Command: $cmd\n"; }
	    $result = `$cmd`;
	    if ($args{verbose}) { print "Result: $result\n"; }
	    if (index($result,"Request has been successfully") > 0) {
		$tok = 1;
	    } else {
		$tok = 0;
	    }
	}
	ok ($tok, "request");
	if (! $args{noclean}) { `rm -fr $tdir`; }
    } elsif ($test eq "retrieve") {
	######################### retrieve, use OU=Services to no pw
	$tok = 1;
	$tdir = "tmp-reqret";
	if (! -d $tdir) {
	    $tok = mkdir($tdir);
	}
	if (! $tok) {
	    ok ( $tok, "Cannot create directory $tdir" );
	    next;
	}
	# check for version ##############################
	$cmd = "../bin/cert-retrieve -V";
	if ($args{verbose}) { print "Command: $cmd\n"; }
	$result = `$cmd`;
	if ($args{verbose}) { print "Result: $result\n"; }
	if ( &count_match($result,$pkgversion) == 3) {
	    $tok = 1;
	} else {
	    $tok = 0;
	}
	ok ($tok, "retrieve version");	
	# do retrieve test ######################
	my $certserial = $testcerts{$CA};
	$tdir = "cert-$certserial";
	$cmd = "../bin/cert-retrieve -ca $CA -dir $tdir -label test ";
	$cmd = $cmd . " -prefix test -serialnum 0x$certserial -noclean";
	$cmd = $cmd . " -p12 $tdir/p12file.p12 -force ";
#	if ($args{noclean}) {
#	    $cmd = $cmd . " -noclean ";
#	}
	if ($args{verbose}) {
	    $cmd = $cmd . " -debug ";
	}
	if ($args{verbose}) { print "Command: $cmd\n"; }
	$result = `$cmd`;
	if ($args{verbose}) { print "Result: $result\n"; }
	if (index($result,"now contain your new credent") > 0) {
	    $tok = 1;
	    $cmd = "openssl x509 -in $tdir/testcert.pem -noout -serial";
	    if ($args{verbose}) { print "Command: $cmd\n"; }
	    $result = `$cmd`;
	    if ($args{verbose}) { print "Result: $result\n"; }
	    if (index($result,$certserial) > 0) {
		$tok = 1;
	    } else {
		$tok = 0;
	    }
	} else {
	    $tok = 0;
	}
	if ( ! -e "$tdir/p12file.p12" ) {
	    $tok = 0;
	    if ($args{verbose}) {
		print "file $tdir/p12file.p12 was not created\n";
	    }
	}
	ok ($tok, "retrieve");
	if (! $args{noclean}) { `rm $tdir/testcert.pem`; }
	#  do clean after successful retrieve
    } elsif ($test eq "renew") {
	######################### renew
	$tok = 1;
	$tdir = "tmp-renew";
	if (! -d $tdir) {
	    $tok = mkdir($tdir);
	}
	if (! $tok) {
	    ok ( $tok, "Cannot create directory $tdir" );
	    next;
	}
	# check for version ##############################
	$cmd = "../bin/cert-renew -V";
	if ($args{verbose}) { print "Command: $cmd\n"; }
	$result = `$cmd`;
	if ($args{verbose}) { print "Result: $result\n"; }
	if ( &count_match($result,$pkgversion) == 3 ) {
	    $tok = 1;
	} else {
	    $tok = 0;
	}
	ok ($tok, "renew version");	
	# do renew test ######################
	my $cdir = $creddir{$args{ca}};
	`cp $cdir/usercert.pem $tdir`;
	`cp $cdir/userkey.pem $tdir`;
	print "This test takes some interactive input.\n";
	print "The passphrase to use is passphrase.\n";
	print "It must be typed rather than pasted with the mouse.\n";
	$cmd = "../bin/cert-renew --ca $CA --dir $tdir  --agree ";
	$cmd = $cmd . " --p12 $tdir/p12file.p12 ";
	if ($args{noclean}) {
	    $cmd = $cmd . " --noclean ";
	}
	if ($args{verbose}) {
	    $cmd = $cmd . " --debug ";
	}
	if ($args{verbose}) { print "Command: $cmd\n"; }
	$result = `$cmd`;
	if ($args{verbose}) { print "Result: $result\n"; }
	$tok = 1;
	$cmd = "openssl x509 -in $tdir/newcert.pem -noout -subject";
	if ($args{verbose}) { print "Command: $cmd\n"; }
	$result = `$cmd`;
	if ($args{verbose}) { print "Result: $result\n"; }
	if (index($result,"Doug Olson") > 0) {
	    $tok = 1;
	} else {
	    $tok = 0;
	}
	if ( ! -e "$tdir/p12file.p12" ) {
	    $tok = 0;
	    if ($args{verbose}) {
		print "file $tdir/p12file.p12 was not created\n";
	    }
	}
	ok ($tok, "renew");
	if (! $args{noclean}) { `rm -fr $tdir`; }
	#  do clean after successful retrieve
    } elsif ($test eq "gridadmin") {
	######################### gridadmin
	$tok = 1;
	$tdir = "tmp-ga";
	if (! -d $tdir) {
	    $tok = mkdir($tdir,0755);
	}
	if (! $tok) {
	    ok ( $tok, "Cannot create directory $tdir" );
	    next;
	}
	# check for version ##############################
	$cmd = "../bin/cert-gridadmin -V";
	if ($args{verbose}) { print "Command: $cmd\n"; }
	$result = `$cmd`;
	if ($args{verbose}) { print "Result: $result\n"; }
	if ( &count_match($result,$pkgversion) == 3 ) {
	    $tok = 1;
	} else {
	    $tok = 0;
	}
	ok ($tok, "gridadmin version");	
	# do gridadmin test ######################
	my $pdir = $creddir{$args{ca}};
	$cmd = "../bin/cert-gridadmin --host polly.wog --service tail ";
	$cmd = $cmd . " --email dlolson\@lbl.gov --affiliation osg --vo osg ";
	$cmd = $cmd . " --prefix pwtest --pdir $pdir --sdir $tdir --ca $CA ";
	if ($args{verbose}) {
	    $cmd = $cmd . " --debug ";
	}
	if ($args{noclean}) {
	    $cmd = $cmd . " --noclean ";
	}
	if ($args{verbose}) {
	    print "Command: $cmd\n";
	    while (my ($evar, $evalue) = each %ENV) {
		print $evar . " = " . $ENV{$evar} . "\n";
	    }
	}
	print "You will be prompted for passhrase (wink wink)\n";
	$result = `$cmd`;
	if ($args{verbose}) {
	    print "Result: $result\n";
	}
	if (index($result,
	   "new certificate and key files are tmp-ga/pwtestcert.pem") > 0) {
	    $tok = 1;
	    $cmd = "openssl x509 -in $tdir/pwtestcert.pem -noout -subject";
	    if ($args{verbose}) {
		print "Command: $cmd\n";
	    }
	    $result = `$cmd`;
	    if ($args{verbose}) {
		print "Result: $result\n";
	    }
	    if (index($result,"tail/polly.wog") > 0) {
		$tok = 1;
	    } else {
		$tok = 0;
	    }
	} else {
	    $tok = 0;
	}
	ok ($tok, "gridadmin");
	if (! $args{noclean}) {
	    `rm -fr $tdir`;
	}
    } elsif ($test eq "multiga") {
	######################### multi-gridadmin
	$tok = 1;
	$tdir = "tmp-multiga";
	if (! -d $tdir) {
	    $tok = mkdir($tdir,0755);
	}
	if (! $tok) {
	    ok ( $tok, "Cannot create directory $tdir" );
	    next;
	}
	# check for version ##############################
	$cmd = "../bin/multi-cert-gridadmin -z";
	if ($args{verbose}) { print "Command: $cmd\n"; }
	$result = `$cmd`;
	if ($args{verbose}) { print "Result: $result\n"; }
	if ( &count_match($result,$pkgversion) == 3 ) {
	    $tok = 1;
	} else {
	    $tok = 0;
	}
	ok ($tok, "multi-gridadmin version");	
	# do multi-gridadmin csr-from-list ######################
	$cmd = "../bin/multi-cert-gridadmin csr-from-list -i cn-list.csv ";
	$cmd = $cmd . " -d tmp-multiga eyes/frog.pond legs/frog.pond";
	if ($args{verbose}) {
	    $cmd = $cmd . " -n ";
	    print "Command: $cmd\n";
	}
	$result = `$cmd`;
	if ($args{verbose}) {
	    print "Result: $result\n";
	}
	if (index($result,"wrote CSR=tmp-multiga/frog.pond-legs-req.pem") > 0)
	{
	    $tok = 1;
	} else {
	    $tok = 0;
	}
	$result = `ls -1 $tdir | grep req.pem | wc | awk '{print $1}' `;
	if ($result != 5) {
	    $tok = 0;
	}
	$result = `ls -1 $tdir | grep key.pem | wc | awk '{print $1}' `;
	if ($result != 5) {
	    $tok = 0;
	}
	ok ($tok,"multi-ga csr-from-list");
	# do multi-gridadmin csr-submit #########################
	my $cdir = $creddir{$args{ca}};
	$cmd = "../bin/multi-cert-gridadmin csr-submit -e dlolson\@lbl.gov ";
	$cmd = $cmd . " -a osg -v osg -c $cdir -s $CA -d $tdir ";
	if ($args{verbose}) {
	    $cmd = $cmd . " -n -x 1 ";
	    print "Command: $cmd\n";
	}
	print "You need to enter passhrase, SSL prompt is missing\n";
	$result = `$cmd`;
	if ($args{verbose}) {
	    print "Result: $result\n";
	}
	$result = `ls -1 $tdir | grep cert.pem | wc | awk '{print $1}' `;
	if ($result != 5) {
	    $tok = 0;
	}
	ok ($tok, "multi-ga csr-submit");
	# do multi-gridadmin csr-from-cert #######################
	# requires csr-submit to be OK
	my $cert1 = "$tdir/frog.pond-eyes-cert.pem";
	my $cert2 = "$tdir/frog.pond-legs-cert.pem";
	if ( ! -r $cert1 || ! -r $cert2) {
	    $tok = 0;
	    if ($args{verbose}) {
		print "missing certs to test for csr-submit, $cert1, $cert2\n";
	    }
	}
	mkdir("$tdir/cfc");
	if ($tok) {
	    my $infile = "$tdir/csr-from-cert-file.csv";
	    open(CFCF,">",$infile);
	    print CFCF "$cert1\n";
	    close(CFCF);
	    my $cdir = $creddir{$args{ca}};
	    $cmd = "../bin/multi-cert-gridadmin csr-from-cert ";
	    $cmd = $cmd . " -e dlolson\@lbl.gov -a osg -v osg -c $cdir ";
	    $cmd = $cmd . " -s $CA -d $tdir/cfc -i $infile $cert2";
	    if ($args{verbose}) {
		$cmd = $cmd . " -n -x 1 ";
		print "Command: $cmd\n";
	    }
	    $result = `$cmd`;
	    if ($args{verbose}) {
		print "Result: $result\n";
	    }
	    if (index($result,"cfc/frog.pond-eyes-req.pem") > 0 &&
		     index($result,"cfc/frog.pond-legs-req.pem")>0) {
		if ($args{verbose}) {
		    print "csr-from-cert is ok\n";
		}
	    } else {
		$tok = 0;
	    }
	}
	ok ($tok, "multi-ga csr-from-cert");
	# do multi-gridadmin cn-submit ##########################
	mkdir ("$tdir/cns");
	my $infile = "$tdir/cn-submit-file.csv";
	open(CNSF,">",$infile);
	print CNSF "legs/frog.pond\n";
	close(CNSF);
	my $cdir = $creddir{$args{ca}};
	$cmd = "../bin/multi-cert-gridadmin cn-submit -e dlolson\@lbl.gov ";
	$cmd = $cmd . " -a osg -v osg -c $cdir -s $CA -d $tdir/cns ";
	$cmd = $cmd . " -i $infile eyes/frog.pond ";
	if ($args{verbose}) {
	    $cmd = $cmd . " -n -x 1 ";
	    print "Command: $cmd\n";
	}
	print "You need to enter passhrase, SSL prompt is missing\n";
	$result = `$cmd`;
	if ($args{verbose}) {
	    print "Result: $result\n";
	}
	$result = `ls -1 $tdir/cns | grep cert.pem | wc | awk '{print $1}' `;
	if ($result != 2) {
	    $tok = 0;
	}	
	ok ($tok,"multi-ga cn-submit");
	if (! $args{noclean}) {
	    `rm -fr $tdir`;
	}
    } else {
	ok ( 1 == 2, "bad test name " . $test);
    }
}
ok ( 2 == 2, "End of tests");

exit(0);

sub usage {
    print "usage: $0 <arguments>
       -h | --help       print this usage and exit
       -c | --ca <name>  name of the CA to test against,
                         testca (amber), and test2 (copper)
       --test <name>     name the script to test, <name>=all will run all tests
                         can be listed many times
       -l | --list       print list of test sets and exit
       --verbose         print extra output
       --noclean         leave any temporary files and directories created

  Peculiarities:
      Must type passphrase for gridadmin test, can not paste it with mouse.
";
}

sub count_match {
    my $instring = shift;
    my $mstring = shift;
    my $cnt = 0;
    my $vpos = 0;
#    print "count_match\n";
#    print " in: $instring\n";
#    print " m: $mstring\n";
    while (($vpos = index($instring,$mstring,$vpos+1)) > 0) {
	$cnt++;
#	print " vpos $vpos, cnt $cnt\n";
    }
    return $cnt;
}
